#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <string>
#include <exception>
#include <vector>
#include <sstream>

using namespace std;

#ifdef WIN32
#include <Winsock2.h>
#define snprintf _snprintf
#else
#define closesocket close
#define SOCKET int
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#endif

#include "uporabno.h"
#include "TipIgre.h"
#include "Karta.h"
#include "igralec.h"

// -------[ socket ]------------------------------------

class TSock {
private:
	SOCKET sock;
	string buffer;
public:
	TSock();
	~TSock();

	bool Connect(char* hostname, int port);
	string readline();
	vector<string> readlineV();
	void writeline(string str);
};

TSock::TSock() : sock(-1), buffer() {}
TSock::~TSock() { closesocket(sock); }

bool TSock::Connect(char *hostname, int port) {
	sockaddr_in saddr;
	struct hostent *he;

#ifdef WIN32
	WORD wVersionRequested = MAKEWORD( 2, 2 );
	WSADATA wsaData;
	WSAStartup(wVersionRequested, &wsaData);
#endif

	he = gethostbyname(hostname);
	if (!he) return false;

	saddr.sin_family = AF_INET;
	saddr.sin_port = htons(port);
	saddr.sin_addr.s_addr = (*(struct in_addr*)he->h_addr).s_addr;
	
	sock = socket(PF_INET, SOCK_STREAM, 0);
	if (sock == -1) return false;
	if (connect(sock, (const sockaddr*)&saddr, sizeof(saddr)) != 0) return false;
	return true;
}

void chop(string &s) {
	if (s[s.length()-1] == '\n') s.erase(s.length()-1);
	if (s[s.length()-1] == '\r') s.erase(s.length()-1);
}

string TSock::readline() {
	string::size_type pos;
	while (true) {
		if ((pos=buffer.find('\n')) != string::npos) {
			string q = buffer.substr(0, pos);
			buffer.erase(0, pos+1);
			chop(q);
			return q;
		} else {
			char tmp[130];
			int s = recv(sock, tmp, 128, 0);
			if (s < 0) throw new string("Napaka pri komunikaciji s streznikom");
			if (s == 0) throw new string("Streznik je zaprl povezavo");
			tmp[s] = '\0';
			buffer += string(tmp);
		}
	}
}

vector<string> TSock::readlineV() {
	string l = readline();
	vector<string> v;
	
	string::size_type fpos, lpos = 0;
	while (lpos < l.length()) {
		fpos = l.find(' ', lpos);
		if (fpos == string::npos) fpos = l.length();
		string x = l.substr(lpos, fpos-lpos);
		v.push_back(x);
		lpos = fpos+1;
	}

	return v;
}
void TSock::writeline(string str) {
	send(sock, str.c_str(), (int)str.length(), 0);
}

// -----------------------------------------------------

void event_loop(TSock &sock) {
	while (true) {
		vector<string> cmda = sock.readlineV();
		if (cmda[0] == "ADIJO") return;
		if (cmda[0] == "KONEC_IGRE") continue;
		if (cmda[0] != "ZACNI") throw new string("Napaka pri komunikaciji s streznikom");
		
		Igralec *t = new Igralec();
		cmda.erase(cmda.begin());
		KartaV karte = stringVKarteN(cmda);
		int pozicija = -1;

		TipIgre predlog = t->zacniIgro(karte);

		sock.writeline(string("PREDLAGAM ") + predlog.toString() + "\n");

		while (true) {
			cmda = sock.readlineV();

			if (cmda[0] == "ADIJO") return;
			if (cmda[0] == "KONEC_IGRE") {
				t->konecIgre(cmda[1]);
				delete t;
				t = NULL;
				pozicija = -1;
				break;
			} else if (cmda[0] == "ZALOZI") {
				int N_grup = atoi(cmda[1].c_str());
				vector<KartaV> talon;
				cmda.erase(cmda.begin(), cmda.begin()+2);
				for (int i=0;i<N_grup;i++) talon.push_back(stringVKarteN(cmda));
				KartaV vzete, zalozene;
				t->zalozi(karte, talon, vzete, zalozene);
				karte = diff<Karta>(conc<Karta>(karte, vzete), zalozene);
				sock.writeline("LEZIM " + KartaVString(vzete) + " " + KartaVString(zalozene) + "\n");
			} else if (cmda[0] == "IGRAJ") {
				pozicija = atoi(cmda[1].c_str());
				int idxGlavnega = atoi(cmda[2].c_str());
				TipIgre tip(cmda[3].c_str());
				KartaV talon_ostanek = stringVKarteN(4, cmda);
				KartaV talon_pobran = stringVKarteN(cmda);
				t->zacniRedniDel(pozicija, idxGlavnega, tip, talon_ostanek, talon_pobran);
				sock.writeline("MHM\n");
			} else if (cmda[0] == "VRZI") {
				int prvi_idx = atoi(cmda[1].c_str());
				KartaV karte_na_mizi = stringVKarteN(2, cmda);
				Karta karta = t->vrziKarto(karte, karte_na_mizi, prvi_idx);
				karte = diff(karte, karta);
				sock.writeline(string("VRZEM ") + karta.toString() + "\n");
			} else if (cmda[0] == "POVZETEK") {
				int prvi_idx = atoi(cmda[1].c_str());
				int zmagovalec_idx = atoi(cmda[2].c_str());
				KartaV karte_na_mizi = stringVKarteN(3, cmda);
				t->konecKroga(pozicija == zmagovalec_idx, prvi_idx, zmagovalec_idx, karte_na_mizi);
				sock.writeline("MHM\n");
			}
		}
	}
}

void connect(char *hostname, TSock &sock) {
	if (!sock.Connect(hostname, 1179)) throw new string("Povezava s streznikom neuspesna");

	string server = sock.readline();
	printf("%s\n", server.c_str());

	sock.writeline(string("AVE ") + Igralec::Ime + "\n");
	string succ = sock.readline();

	if (succ != "PRIJAVA USPESNA") throw new string((string("Prijava na streznik neuspesna: ") + succ).c_str());
}

int main(int argc, char* argv[]) {
	char *hostname;
	TSock sock;

	if (argc < 2) hostname = "localhost";
	else hostname = argv[1];

	try {
		connect(hostname, sock);
		event_loop(sock);
	} catch (string *e) {
		printf("%s\n", e->c_str());
	}

	return 0;
}
