#include <sstream>
#include <algorithm>
#include <functional>
#include "uporabno.h"
#include "Karta.h"

using namespace std;

namespace barva {
	string vString(Barva barva) {
        switch (barva) {
			case barva::TAROK: return "T";
			case barva::KARO: return "K";
			case barva::KRIZ: return "R";
			case barva::PIK: return "P";
			case barva::SRCE: return "S";
			case barva::NEZNANA: return "X";
			default: return "X";
        }
	}
	
	Barva vBarvo(char c) {
		switch (c) {
			case 'T': return barva::TAROK;
			case 'K': return barva::KARO;
			case 'R': return barva::KRIZ;
			case 'P': return barva::PIK;
			case 'S': return barva::SRCE;
			default: return barva::NEZNANA;
		}
	}
};

string intToStr(const int x) {
	std::stringstream ss;
	ss << x;
	return ss.str();
}

int steviloTock(const KartaV &karte) {
	int sum = 0;
	for (unsigned int i=0;i<karte.size();i++) sum += karte[i].stTock();
	return sum;
}

string KartaVString(const KartaV &karte) {
	string ret = intToStr(karte.size());
	for (unsigned int i=0;i<karte.size();i++) {
		ret += " " + karte[i].toString();
	}
	return ret;
}

KartaV stringVKarteN(int slice, vector<string> &s) {
	s.erase(s.begin(), s.begin()+slice);
	return stringVKarteN(s);
}

KartaV stringVKarteN(vector<string> &s) {
	int N = atoi(s[0].c_str());
	KartaV vk;
	for (int i=0;i<N;i++) vk.push_back(Karta(s[i+1]));
	s.erase(s.begin(), s.begin() + N + 1);
	return vk;
}

struct niEnakeBarve : public std::unary_function<Karta, bool> {
	Barva cb;

	niEnakeBarve(const Barva icb) : cb(icb) {}

	bool operator()(const Karta &k) const {
		return k.barva != cb;
	}
};

KartaV veljavnePoteze(const KartaV &mojeKarte, const KartaV &karteNaMizi) {
	if (karteNaMizi.size() == 0) return mojeKarte;

	KartaV enakeBarve(mojeKarte.size());
	KartaV::iterator e = remove_copy_if(mojeKarte.begin(), mojeKarte.end(), enakeBarve.begin(), niEnakeBarve(karteNaMizi[0].barva));
	enakeBarve.erase(e,enakeBarve.end());
	if (enakeBarve.size() > 0) return enakeBarve;
	
	KartaV taroki(mojeKarte.size());
	KartaV::iterator f = remove_copy_if(mojeKarte.begin(), mojeKarte.end(), taroki.begin(), niEnakeBarve(barva::TAROK));
	taroki.erase(f,taroki.end());
	if (taroki.size() > 0) return taroki;

	return mojeKarte;
}
